"use strict";
const ts = require("typescript");
const fs_extra_p_1 = require("fs-extra-p");
const markdown = require("markdown-it");
const __awaiter = require("./awaiter");
function generateDocs(program) {
    const topicToProperties = new Map();
    for (const sourceFile of program.getSourceFiles()) {
        if (sourceFile.isDeclarationFile) {
            continue;
        }
        for (const statement of sourceFile.statements) {
            if (statement.kind === ts.SyntaxKind.InterfaceDeclaration) {
                const description = getComment(statement, sourceFile);
                if (description == null) {
                    continue;
                }
                const lineBreakIndex = description.indexOf("\n");
                const header = description.substring(0, lineBreakIndex == -1 ? description.length : lineBreakIndex).replace(/`/g, "");
                const interfaceDeclaration = statement;
                const interfaceName = interfaceDeclaration.name.text;
                const interfaceDescriptor = {
                    name: interfaceName,
                    description: description,
                    header: header,
                };
                let nameToProperty = topicToProperties.get(interfaceDescriptor);
                for (const member of interfaceDeclaration.members) {
                    if (member.kind === ts.SyntaxKind.PropertySignature) {
                        let comment = getComment(member, sourceFile);
                        if (comment == null) {
                            continue;
                        }
                        if (nameToProperty == null) {
                            nameToProperty = new Map();
                            topicToProperties.set(interfaceDescriptor, nameToProperty);
                        }
                        const symbol = member.symbol;
                        let isOptional = symbol != null && (symbol.flags & ts.SymbolFlags.Optional) !== 0;
                        if (isOptional) {
                            comment = comment
                                .split("\n")
                                .map(it => it.trim())
                                .filter(it => {
                                if (it.startsWith("@required")) {
                                    isOptional = false;
                                    return false;
                                }
                                return true;
                            })
                                .join("\n");
                        }
                        nameToProperty.set(member.name.text, new PropertyDescriptor(interfaceName, comment, isOptional));
                    }
                }
            }
        }
    }
    return topicToProperties;
}
exports.generateDocs = generateDocs;
function writeDocFile(docOutFile, content) {
    return __awaiter(this, void 0, void 0, function* () {
        let existingContent;
        try {
            existingContent = yield fs_extra_p_1.readFile(docOutFile, "utf8");
        }
        catch (e) {
        }
        console.log(`Write doc to ${docOutFile}`);
        if (existingContent == null) {
            return fs_extra_p_1.outputFile(docOutFile, content);
        }
        else {
            const startMarker = "<!-- do not edit. start of generated block -->";
            const endMarker = "<!-- end of generated block -->";
            const start = existingContent.indexOf(startMarker);
            const end = existingContent.indexOf(endMarker);
            if (start != -1 && end != -1) {
                return fs_extra_p_1.outputFile(docOutFile, existingContent.substring(0, start + startMarker.length) + "\n" + content + "\n" + existingContent.substring(end));
            }
            else {
                return fs_extra_p_1.outputFile(docOutFile, content);
            }
        }
    });
}
exports.writeDocFile = writeDocFile;
function stripIndent(str) {
    const match = str.match(/^[ \t]*(?=\S)/gm);
    const indent = match == null ? 0 : Math.min.apply(Math, match.map(it => it.length));
    return indent > 0 ? str.replace(new RegExp("^[ \\t]{" + indent + "}", "gm"), "") : str;
}
function getComment(node, sourceFile) {
    const leadingCommentRanges = ts.getLeadingCommentRanges(sourceFile.text, node.pos);
    if (leadingCommentRanges == null || leadingCommentRanges.length === 0) {
        return null;
    }
    else {
        const commentRange = leadingCommentRanges[0];
        if (sourceFile.text[commentRange.pos] == "/" && sourceFile.text[commentRange.pos + 1] == "/") {
            return null;
        }
        else {
            const isTwo = sourceFile.text[commentRange.pos + 2] == "*";
            return stripIndent(sourceFile.text.slice(commentRange.pos + (isTwo ? 3 : 2), commentRange.end - "*/".length)).trim();
        }
    }
}
function writeToJs(program) {
    let result = "";
    for (const sourceFile of program.getSourceFiles()) {
        if (!sourceFile.isDeclarationFile) {
            continue;
        }
        for (const statement of sourceFile.statements) {
            if (statement.kind === ts.SyntaxKind.InterfaceDeclaration) {
                let description = getComment(statement, sourceFile);
                if (description == null) {
                    continue;
                }
                if (description.startsWith("#")) {
                    const nextLineIndex = description.indexOf("\n");
                    if (nextLineIndex > 0) {
                        description = description.substring(nextLineIndex + 1).trim();
                    }
                }
                const interfaceDeclaration = statement;
                const interfaceName = interfaceDeclaration.name.text;
                result += `/**\n  * ${description}\n  */\n`;
                result += `class ${interfaceName} {\n`;
                for (const member of interfaceDeclaration.members) {
                    if (member.kind === ts.SyntaxKind.PropertySignature) {
                        const comment = getComment(member, sourceFile);
                        if (comment != null) {
                            result += `  /**\n  * ${comment}\n  */\n`;
                        }
                        const symbol = member.symbol;
                        result += `  ${member.name.text}\n`;
                    }
                }
                result += "}\n\n";
            }
        }
    }
    return result.trim();
}
exports.writeToJs = writeToJs;
function renderDocs(topicToProperties) {
    let result = "";
    const md = markdown({
        typographer: true,
    })
        .disable(["link", "emphasis"]);
    function render(src) {
        return src.includes("\n") ? md.render(src).trim().replace(/\n/g, " ") : src;
    }
    const keys = Array.from(topicToProperties.keys());
    let w = 0;
    let subW = 0;
    const topLevelKeys = new Map();
    for (const interfaceDescriptor of keys) {
        const header = interfaceDescriptor.header;
        if (header.startsWith("# ")) {
            if (!topLevelKeys.has(header)) {
                topLevelKeys.set(header, w += 1000);
            }
        }
        else {
            let key = header.split(".")[1];
            if (!topLevelKeys.has(key)) {
                topLevelKeys.set(key, subW += 100000);
            }
        }
    }
    function headerWeight(text) {
        let weight = 0;
        for (let k of topLevelKeys.keys()) {
            if (text.endsWith(`.${k}`) || text.includes(`.${k}.`)) {
                weight = topLevelKeys.get(k);
                break;
            }
        }
        for (let i = 0; i < text.length; i++) {
            if (text[i] == "#") {
                weight += 1;
            }
            else {
                break;
            }
        }
        return weight;
    }
    keys.sort(function (a, b) {
        const n1 = a.header;
        const n2 = b.header;
        const hDiff = headerWeight(n1) - headerWeight(n2);
        if (hDiff != 0) {
            return hDiff;
        }
        return n1.localeCompare(n2);
    });
    let tocHeaderOffset = 0;
    for (const interfaceDescriptor of keys) {
        let header = interfaceDescriptor.header;
        for (let i = 0; i < header.length; i++) {
            if (header[i] != "#") {
                if (i > 0) {
                    if (result === "") {
                        tocHeaderOffset = i;
                    }
                    result += "  ".repeat(i - tocHeaderOffset);
                }
                result += "* ";
                header = `[${header.substring(i).trim()}](#${interfaceDescriptor.name})`;
                break;
            }
        }
        result += header + "\n";
    }
    for (const interfaceDescriptor of keys) {
        const nameToProperty = topicToProperties.get(interfaceDescriptor);
        result += `\n${anchor(interfaceDescriptor.name)}\n${interfaceDescriptor.description}\n`;
        if (interfaceDescriptor.description.includes("\n")) {
            result += "\n";
        }
        result += "| Name | Description\n";
        result += "| --- | ---";
        nameToProperty.forEach((descriptor, propertyName) => {
            let bold = descriptor.isOptional ? "" : "**";
            result += `\n| ${bold}${propertyName}${bold} | `;
            result += anchor(descriptor.interfaceName + "-" + propertyName);
            result += render(descriptor.description);
        });
        result += "\n";
    }
    return result;
}
exports.renderDocs = renderDocs;
function anchor(link) {
    return `<a name="${link}"></a>`;
}
class InterfaceDescriptor {
}
exports.InterfaceDescriptor = InterfaceDescriptor;
class PropertyDescriptor {
    constructor(interfaceName, description, isOptional) {
        this.interfaceName = interfaceName;
        this.description = description;
        this.isOptional = isOptional;
    }
}
exports.PropertyDescriptor = PropertyDescriptor;
//# sourceMappingURL=docGenerator.js.map