"use strict";
const ts = require("typescript");
const path = require("path");
const fs_extra_p_1 = require("fs-extra-p");
const __awaiter = require("./awaiter");
function transpile(transpilator) {
    return __awaiter(this, void 0, void 0, function* () {
        const paths = process.argv.slice(2);
        if (paths.length == 0) {
            paths.push(process.cwd());
        }
        return transpilePaths(paths, transpilator);
    });
}
exports.transpile = transpile;
function transpilePaths(paths, transpilator) {
    return __awaiter(this, void 0, void 0, function* () {
        for (const basePath of paths) {
            try {
                yield build(basePath, transpilator);
            }
            catch (e) {
                if (!(e instanceof CompilationError)) {
                    throw e;
                }
                for (const diagnostic of e.errors) {
                    if (diagnostic.file == null) {
                        console.log(diagnostic.messageText);
                        continue;
                    }
                    const location = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
                    const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
                    console.log(`${diagnostic.file.fileName} (${location.line + 1}, ${location.character + 1}): ${message}`);
                }
                process.exit(-1);
                return;
            }
        }
    });
}
exports.transpilePaths = transpilePaths;
function build(basePath, transpilator) {
    return __awaiter(this, void 0, void 0, function* () {
        const tsConfigPath = path.join(basePath, "tsconfig.json");
        const jsonResult = ts.parseConfigFileTextToJson(tsConfigPath, yield fs_extra_p_1.readFile(tsConfigPath, "utf8"));
        if (jsonResult.error != null) {
            throw new CompilationError([jsonResult.error]);
        }
        const result = ts.parseJsonConfigFileContent(jsonResult.config, ts.sys, basePath);
        checkErrors(result.errors);
        yield transpilator(basePath, result, jsonResult.config);
    });
}
function checkErrors(errors) {
    if (errors.length !== 0) {
        throw new CompilationError(errors);
    }
}
exports.checkErrors = checkErrors;
class CompilationError extends Error {
    constructor(errors) {
        super("Compilation error");
        this.errors = errors;
    }
}
//# sourceMappingURL=util.js.map